//*********************** 2652vi.c ****************************
//* This file contains Lab View DLL to communicate with
//* Sensoray model 2652 relay board.
//* Written by Alexander Kostromitin
//* Copyright (C) Sensoray Company 2006
//*************************************************************


#define DEBUG     1     // generate some additional info


#include <time.h>
#include <string.h>
#include <stdio.h>
#include <windows.h>
#include <commdlg.h>    // includes common dialog functionality
#include <commctrl.h>
#include <dlgs.h>       // includes common dialog template defines
#include <cderr.h>      // includes the common dialog error codes
#include <io.h>
#include <math.h>


//#include "Win2600.h"
#include "f2600.h"


static int dllinit(void);
static void dllclose(void);
static int findmodule(int mm, int id, int from);
static int io_exec(void *x);
static void error(char *fmt, ... );



#define chkerr(f)   { int err=(f); if(err) error("Error #%x", err); }


// Common parameters
static char errstr[256];                // error message string


// DLL entry point
BOOL WINAPI DllEntryPoint(HANDLE hDLL, DWORD dwReason, LPVOID lpReserved)
{
  if(dwReason == DLL_PROCESS_ATTACH)
    return dllinit() == 0;
  if(dwReason == DLL_PROCESS_DETACH)
    dllclose();
  return 1;
}

// alternative entry point
BOOL WINAPI DllMain(HANDLE hDLL, DWORD dwReason, LPVOID lpReserved)
{
  if(dwReason == DLL_PROCESS_ATTACH)
    return dllinit() == 0;
  if(dwReason == DLL_PROCESS_DETACH)
    dllclose();
  return 1;
}



/////////////////////////////////////////////////////////////////
// exported functions
//
// note: for compatibility reasons all exported functions
// declared as __stdcall (WINAPI), do not use regular C
// convention to call them from VI.
/////////////////////////////////////////////////////////////////


// Open board
//  ClientDotAdrs: IP address of host computer's connection (use "0" for default)
//  MMDotAdrs: IP address of Main Module (2601)
//  mm: index of registering Main Module in internal DLL's table
//  iom: module (2601 IOM connector) number (use -1 to open first module found)
//  wdmsec: watcdog interval in milliseconds (maximum time between commands to
//   avoid internal reset; use 0 to disable)
__declspec (dllexport) int __stdcall openboard(char *ClientDotAdrs, char *MMDotAdrs, int mm, int iom, int wdmsec)
{
  int err;
  void *x;
  int port;

  // establish connection with 2601 base module
  S26_BoardOpen(mm, ClientDotAdrs, MMDotAdrs);
  err = !S26_ResetNetwork(mm);
  if(err)
    {
    error("Can't initialize the MM");
    return(-1);
    }
  // set watchdog
  if(wdmsec != 0)
    {
    wdmsec /= 100;
    if(wdmsec <= 0)
      wdmsec = 1;
    else if(wdmsec > 255)
      wdmsec = 255;
    }
  x = S26_SchedOpen(mm, 1);
  S26_Sched2601_SetWatchdog(x, wdmsec);
  io_exec(x);

  // find 1st board
  port = findmodule(mm, 2652, iom);
  if(port < 0)
    {
    error("No board found");
    return(-1);
    }
  return(port);
}


// Get last error message
// Clear error status if required
__declspec (dllexport) char *__stdcall getlasterror(int clear)
{
  static char err[256];

  strcpy(err, errstr);
  if(clear)
    strcpy(errstr, "Ok");
  return err;
}



//////////////////////////////////////////////////////////////////////
// Board specific functions.
// See Model 2600 Family Programming Guide for details.
//////////////////////////////////////////////////////////////////////


// Return the measured states of all eight relay coil drivers
//  mm: index of the Main Module used in <openboard>
//  iom: module (2601 IOM connector) number
//	return: physical state of the relay coil drivers
__declspec (dllexport) int __stdcall getinputs(int mm, int iom)
{
  void *x;                // Transaction object.
  char sts;

  x = S26_SchedOpen(mm, 1);
  chkerr(S26_Sched2652_GetInputs(x, iom, &sts));
  io_exec(x);
  return sts;
}



// Set all eight relays
//  mm: index of the Main Module used in <openboard>
//  iom: module (2601 IOM connector) number
__declspec (dllexport) void __stdcall setoutputs(int mm, int iom, int state)
{
  void *x;                // Transaction object.
  char sts = state;

  x = S26_SchedOpen(mm, 1);
  chkerr(S26_Sched2652_SetOutputs(x, iom, &sts));
  io_exec(x);
}



// Set output mode of all channels
//  mm: index of the Main Module used in <openboard>
//  iom: module (2601 IOM connector) number
//  modes: output modes (one bit per channel, 0 - standard, 1 - PWM)
__declspec (dllexport) void __stdcall setoutmodes(int mm, int iom, int modes)
{
  char m = (char)modes;
  void *x;                // Transaction object.

  x = S26_SchedOpen(mm, 1);
  S26_Sched2652_SetModes(x, iom, &m);
  io_exec(x);
}


// Set PWM ratio for a channel
//  mm: index of the Main Module used in <openboard>
//  iom: module (2601 IOM connector) number
//  cnl: channel
//  ontime: PWM ON time
//  offtime: PWM OFF time
__declspec (dllexport) void __stdcall setoutpwm(int mm, int iom, int cnl, int ontime, int offtime)
{
  void *x;                // Transaction object.

  x = S26_SchedOpen(mm, 1);
  S26_Sched2652_SetPwmRatio(x, iom, cnl, ontime, offtime);
  io_exec(x);
}



/////////////////////////////////////////////////////////////////
// internal functions
/////////////////////////////////////////////////////////////////

// Open and init main DLL
static int dllinit(void)
{
  int err;

  // open DLL
  #if DEBUG
  // local directory
  //err = S26_DLLOpen("d:\\ntprog\\windrv\\2600\\labview\\2600demo", 1);
  err = S26_DLLOpen("c:\\2600\\labview", 1);
  #else
  // some common directory (windows\system32)
  err = S26_DLLOpen(NULL, 1);
  #endif

  if(err)
    {
    error("2600 DLL open error #%d", err);
    return(-1);
    }

  return(0);
}

                                                            
// Close main DLL
static void dllclose(void)
{
  S26_DriverClose();
}


// Find a module
//  id: module ID
//  from: number of module to start from
static int findmodule(int mm, int id, int from)
{
  int err;
  int iom;
  unsigned short num;
  unsigned short ioms[16];
  unsigned char sts[16];

  if(from < 0)
    {
    err = S26_RegisterAllIoms(mm, 1000, &num, ioms, sts, 1);
    if(err)
      return(-1);
    from = 0;
    }
  // Count the connected IOMs, stop on first board found.
  for(iom = from; iom < 16; iom++)
    {
    if(ioms[iom] == id)
      {
      return(iom);
      }
    }
  return(-1);
}


// Error message
static void error(char *fmt, ... )
{
  va_list argptr;

  va_start(argptr, fmt);
  vsprintf(errstr, fmt, argptr);
  va_end(argptr);
  // print error messages to file
  #if DEBUG
    {
    FILE *f;

    if((f = fopen("c:\\2652vidll.log", "a")) == NULL)
      return;
    fprintf(f, "%s\n", errstr);
    fclose(f);
    }
  #endif

}


// Execute all scheduled i/o.
// Returns zero if successful.
static int io_exec(void *x)
{
  char status[16];		// Iom status info.
  GWERR err;

  // Execute the scheduled i/o.  Report error if one was detected.
  err = S26_SchedExecute(x, 1000, status );
  if(err)
    error("Connection error 0x%08x", err);
  return(err);
}






